@@ -42,7 +42,7 @@ def main(sys_args: Optional[List[str]] = None) -> None:
         "-b",
         "--bind",
         dest="binds",
-        help=""" The host/address to bind to. Should be either host:port, host,
+        help=""" The TCP host/address to bind to. Should be either host:port, host,
         unix:path or fd://num, e.g. 127.0.0.1:5000, 127.0.0.1,
         unix:/tmp/socket or fd://33 respectively.  """,
         default=[],
@@ -72,6 +72,9 @@ def main(sys_args: Optional[List[str]] = None) -> None:
         help="The target location for the error log, use `-` for stderr",
         default=sentinel,
     )
+    parser.add_argument(
+        "-g", "--group", help="Group to own any unix sockets.", default=sentinel, type=int
+    )
     parser.add_argument(
         "-k",
         "--worker-class",
@@ -91,7 +94,7 @@ def main(sys_args: Optional[List[str]] = None) -> None:
     parser.add_argument(
         "--insecure-bind",
         dest="insecure_binds",
-        help="""The host/address to bind to. SSL options will not apply to these binds.
+        help="""The TCP host/address to bind to. SSL options will not apply to these binds.
         See *bind* for formatting options. Care must be taken! See HTTP -> HTTPS redirection docs.
         """,
         default=[],
@@ -103,6 +106,15 @@ def main(sys_args: Optional[List[str]] = None) -> None:
     parser.add_argument(
         "-p", "--pid", help="Location to write the PID (Program ID) to.", default=sentinel
     )
+    parser.add_argument(
+        "--quic-bind",
+        dest="quic_binds",
+        help="""The UDP/QUIC host/address to bind to. See *bind* for formatting
+        options.
+        """,
+        default=[],
+        action="append",
+    )
     parser.add_argument(
         "--reload",
         help="Enable automatic reloads on code changes",
@@ -116,6 +128,16 @@ def main(sys_args: Optional[List[str]] = None) -> None:
         "--statsd-host", help="The host:port of the statsd server", default=sentinel
     )
     parser.add_argument("--statsd-prefix", help="Prefix for all statsd messages", default="")
+    parser.add_argument(
+        "-m",
+        "--umask",
+        help="The permissions bit mask to use on any unix sockets.",
+        default=sentinel,
+        type=int,
+    )
+    parser.add_argument(
+        "-u", "--user", help="User to own any unix sockets.", default=sentinel, type=int
+    )
 
     def _convert_verify_mode(value: str) -> ssl.VerifyMode:  # type: ignore
         try:
@@ -172,6 +194,8 @@ def main(sys_args: Optional[List[str]] = None) -> None:
         config.errorlog = args.error_log
     if args.error_logfile is not sentinel:
         config.errorlog = args.error_log
+    if args.group is not sentinel:
+        config.group = args.group
     if args.keep_alive is not sentinel:
         config.keep_alive_timeout = args.keep_alive
     if args.keyfile is not sentinel:
@@ -186,6 +210,10 @@ def main(sys_args: Optional[List[str]] = None) -> None:
         config.statsd_host = args.statsd_host
     if args.statsd_prefix is not sentinel:
         config.statsd_prefix = args.statsd_prefix
+    if args.umask is not sentinel:
+        config.umask = args.umask
+    if args.user is not sentinel:
+        config.user = args.user
     if args.worker_class is not sentinel:
         config.worker_class = args.worker_class
     if args.workers is not sentinel:
@@ -195,6 +223,8 @@ def main(sys_args: Optional[List[str]] = None) -> None:
         config.bind = args.binds
     if len(args.insecure_binds) > 0:
         config.insecure_bind = args.insecure_binds
+    if len(args.quic_binds) > 0:
+        config.quic_bind = args.quic_binds
 
     run(config)
 
