@@ -2,19 +2,24 @@ import asyncio
 import platform
 import signal
 import ssl
+from functools import partial
 from multiprocessing.synchronize import Event as EventType
-from typing import Any, Coroutine, Optional
+from os import getpid
+from socket import socket
+from typing import Any, Awaitable, Callable, Coroutine, Optional
 
 from .lifespan import Lifespan
-from .server import Server
 from .statsd import StatsdLogger
+from .tcp_server import TCPServer
+from .udp_server import UDPServer
 from ..config import Config, Sockets
 from ..typing import ASGIFramework
 from ..utils import (
-    check_shutdown,
+    check_multiprocess_shutdown_event,
     load_application,
     MustReloadException,
     observe_changes,
+    raise_shutdown,
     repr_socket_addr,
     restart,
     Shutdown,
@@ -37,12 +42,21 @@ async def _windows_signal_support() -> None:
         await asyncio.sleep(1)
 
 
+def _share_socket(sock: socket) -> socket:
+    # Windows requires the socket be explicitly shared across
+    # multiple workers (processes).
+    from socket import fromshare  # type: ignore
+
+    sock_data = sock.share(getpid())  # type: ignore
+    return fromshare(sock_data)
+
+
 async def worker_serve(
     app: ASGIFramework,
     config: Config,
     *,
     sockets: Optional[Sockets] = None,
-    shutdown_event: Optional[EventType] = None,
+    shutdown_trigger: Optional[Callable[..., Awaitable[None]]] = None,
 ) -> None:
     config.set_statsd_logger_class(StatsdLogger)
 
@@ -80,8 +94,8 @@ async def worker_serve(
 
         tasks.append(loop.create_task(_check_signal()))
 
-    if shutdown_event is not None:
-        tasks.append(loop.create_task(check_shutdown(shutdown_event, asyncio.sleep)))
+    if shutdown_trigger is not None:
+        tasks.append(loop.create_task(raise_shutdown(shutdown_trigger)))
 
     if config.use_reloader:
         tasks.append(loop.create_task(observe_changes(asyncio.sleep)))
@@ -92,10 +106,13 @@ async def worker_serve(
         ssl_handshake_timeout = config.ssl_handshake_timeout
 
     async def _server_callback(reader: asyncio.StreamReader, writer: asyncio.StreamWriter) -> None:
-        await Server(app, loop, config, reader, writer)
+        await TCPServer(app, loop, config, reader, writer)
 
     servers = []
     for sock in sockets.secure_sockets:
+        if config.workers > 1 and platform.system() == "Windows":
+            sock = _share_socket(sock)
+
         servers.append(
             await asyncio.start_server(
                 _server_callback,
@@ -110,6 +127,9 @@ async def worker_serve(
         await config.log.info(f"Running on {bind} over https (CTRL + C to quit)")
 
     for sock in sockets.insecure_sockets:
+        if config.workers > 1 and platform.system() == "Windows":
+            sock = _share_socket(sock)
+
         servers.append(
             await asyncio.start_server(
                 _server_callback, backlog=config.backlog, loop=loop, sock=sock
@@ -118,6 +138,14 @@ async def worker_serve(
         bind = repr_socket_addr(sock.family, sock.getsockname())
         await config.log.info(f"Running on {bind} over http (CTRL + C to quit)")
 
+    for sock in sockets.quic_sockets:
+        if config.workers > 1 and platform.system() == "Windows":
+            sock = _share_socket(sock)
+
+        await loop.create_datagram_endpoint(lambda: UDPServer(app, loop, config), sock=sock)
+        bind = repr_socket_addr(sock.family, sock.getsockname())
+        await config.log.info(f"Running on {bind} over quic (CTRL + C to quit)")
+
     reload_ = False
     try:
         gathered_tasks = asyncio.gather(*tasks)
@@ -146,8 +174,13 @@ def asyncio_worker(
     config: Config, sockets: Optional[Sockets] = None, shutdown_event: Optional[EventType] = None
 ) -> None:
     app = load_application(config.application_path)
+
+    shutdown_trigger = None
+    if shutdown_event is not None:
+        shutdown_trigger = partial(check_multiprocess_shutdown_event, shutdown_event, asyncio.sleep)
+
     _run(
-        worker_serve(app, config, sockets=sockets, shutdown_event=shutdown_event),
+        worker_serve(app, config, sockets=sockets, shutdown_trigger=shutdown_trigger),
         debug=config.debug,
     )
 
@@ -163,8 +196,13 @@ def uvloop_worker(
         asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
 
     app = load_application(config.application_path)
+
+    shutdown_trigger = None
+    if shutdown_event is not None:
+        shutdown_trigger = partial(check_multiprocess_shutdown_event, shutdown_event, asyncio.sleep)
+
     _run(
-        worker_serve(app, config, sockets=sockets, shutdown_event=shutdown_event),
+        worker_serve(app, config, sockets=sockets, shutdown_trigger=shutdown_trigger),
         debug=config.debug,
     )
 
