@@ -1,4 +1,5 @@
 import warnings
+from typing import Awaitable, Callable, Optional
 
 import trio
 
@@ -11,6 +12,7 @@ async def serve(
     app: ASGIFramework,
     config: Config,
     *,
+    shutdown_trigger: Optional[Callable[..., Awaitable[None]]] = None,
     task_status: trio._core._run._TaskStatus = trio.TASK_STATUS_IGNORED,
 ) -> None:
     """Serve an ASGI framework app given the config.
@@ -26,12 +28,15 @@ async def serve(
     this function, therefore configuration values that relate to loop
     setup or process setup are ignored.
 
+    Arguments:
+        app: The ASGI application to serve.
+        config: A Hypercorn configuration object.
+        shutdown_trigger: This should return to trigger a graceful
+            shutdown.
     """
     if config.debug:
         warnings.warn("The config `debug` has no affect when using serve", Warning)
     if config.workers != 1:
         warnings.warn("The config `workers` has no affect when using serve", Warning)
-    if config.worker_class != "trio":
-        warnings.warn("The config `worker_class` has no affect when using serve", Warning)
 
-    await worker_serve(app, config, task_status=task_status)
+    await worker_serve(app, config, shutdown_trigger=shutdown_trigger, task_status=task_status)
